//*******************************************************************************************//
//*																							*//
//*		Project :	RC - Servo tester														*//
//*		Device	:	PSoC 4 Cy8C4245AXI483													*//
//*		Version	:	1.0																		*//
//*																							*//
//*		Author	:	Bob Marlowe																*//
//*		(C) 2014 Jörg Meier Software - Entwicklung											*//
//*		Steinfelder Straße 11																*//
//*		D-27404 Brümmerhof																	*//
//*		Germany																				*//
//*																							*//
//*		briefe@jmeiersoftware.de															*//
//*																							*//
//*******************************************************************************************//
//*																							*//
//*		Purpose	:	Generating the signals for up to eight (8) rc-servos					*//
//*					Project runs on Pioneer board as well as on a CY8CKIT-049-42xx			*//
//*																							*//
//*																							*//
//*		Inputs	:	8 Analog signals beteen 0V and VDDA										*//
//*																							*//
//*		Outputs	:	8 PWM signals in the range of 1 to 2ms repeated every 22.5ms			*//
//*					1 Multiplex (PPM) sum-signal  											*//
//*																							*//
//*					An additional sync-output (heartbeat) at 22.5ms							*//
//*					A signal triggered at SOC (start of conversion) before the interrupt	*//
//*					handler is entered and cleared with the last instruction before the		*//
//*					handler exits, so the time within can be measured.						*//
//*																							*//
//*																							*//
//*		Both the PWM and the MPX signal could be used directly by servos when 				*//
//*		PSoC and servos are operated at 4.8V 												*//
//*																							*//
//*		This design, hardware and software is provided as-it-is as a proof-of-concept.		*//
//*		Use at own risk, no liabilities.													*//
//*																							*//
//*																							*//
//*******************************************************************************************//
//*																							*//
//*							Principals of operation											*//
//*																							*//
//*		One PWM and four TCPWM are used to generate the required signals.					*//
//*		The interrupt of the PWM is used to trigger ADC-conversion during the time where	*//
//*		all TCPWMs are stopped, so that there is no influence when writing new 				*//
//*		period values. The ADC is configured for single-shot and will generate an 			*//
//*		interrupt at the end of conversion of all inputs.									*//
//*		Since the TCPWMs must be halted there appeared one glitch at the outputs which		*//
//*		had to be eliminated with 4 glitchfilter modules. Thanks to the architecture of		*//
//*		the PSoC 4 everything fits into and there are still reserves for changing the 		*//
//*		polarity of the output-signals when needed.											*//
//*																							*//
//*******************************************************************************************//

//*******************************************************************************************//
//******************************* Libraries *************************************************//
//*******************************************************************************************//
#include <project.h>

//*******************************************************************************************//
//*************************** Definitions and Macros ****************************************//
//*******************************************************************************************//
#define forever			1										// I like that!
#define	NumServos		8										// number of servos 
#define ADC_MAX			0x07ff									// 12-bit unsigned 
#define Period2ms		4800									// Period value to have a 2ms pulse width (Servo maximum)
#define Period1ms		2400									// Period value to have a 1ms pulse width (Servo minimum)
#define Pause			Period1ms * 3 / 10						// Width of 0.3ms pulse

#define PulseStart		0b001									// Control bit to start PulseGen Timer
#define ADCStart		0b010									// Control bit to start ADC conversion cycle
#define MPXFire			0b100									// Control bit to fire an MPX-Pulse

//*******************************************************************************************//
//****************** Structures, Unions and Enums *******************************************//
//*******************************************************************************************//

//*******************************************************************************************//
//*************************** Global Data Area **********************************************//
//*******************************************************************************************//
uint8 ChannelValues[NumServos] = { 70,20,40,90,99,90,40,10};	// Values in percent of full-scale
uint8 CurrentChannel = 0;
//*******************************************************************************************//
//******************************* Prototypes ************************************************//
//*******************************************************************************************//
CY_ISR_PROTO(ADCHandler);
CY_ISR_PROTO(ChannelHandler);
CY_ISR_PROTO(MainHandler);
//*******************************************************************************************//
//******************************* Program Area **********************************************//
//*******************************************************************************************//
void InitializeHardware(void)									// Nomen est omen
{
	CyGlobalIntEnable;											// Interrupts on

	
	ADCInt_StartEx(ADCHandler);									// Interrupt from ADC
	ADC_Start();												// Now start ADC
	
//	Counter_Start();											// Servo-channel counter
	IntChannel_StartEx(ChannelHandler);							// and the interrupt handler
	
	PulseGen_Start();											// PWM for generating the pulses
	MPXGen_Start();												// PWM for pulse sharpening
	
	MainTimer_Start();											// 22.5 ms Timer
	Int20ms_StartEx(MainHandler);								// and the associated interrupt
}
//*******************************************************************************************//

int main()														//
{
	InitializeHardware();										// Setup all modules

	while(forever)												// Main Loop
    {
       															// No CPU used in main-loop, all done in interrupt handlers
    }
}
//*******************************************************************************************//

uint32 PercentToServoPulse(int16 Percent)						// Delivers the period for the TCPWMs
{
uint32	Result;
	Result = ((uint32)Percent * Period1ms) / 100 + Period1ms -50;
	return Result;
}
//*******************************************************************************************//

uint32 CountsToPercent(uint16 Count)							// Calculate from ADC to 0..100%
{
	return ((uint32)Count * 100u) / ADC_MAX;
}

void SetupPulse(uint8 Channel)									//  Program the MPX Pulse
{
uint16	Period;
	if(Channel < NumServos)
	{
		Period = PercentToServoPulse(ChannelValues[Channel]);	// Calculate new period
	}
	else
	{
		Period = Pause;											// Final pulse before the sync-pause
	}
	PulseGen_WritePeriod(Period);								// Write new pulse length
	System_Control_Write(PulseStart);							// Fire new pulse
}
//*******************************************************************************************//

CY_ISR(ADCHandler)												// Conversion is done, retrieve all values
{
uint32	Channel;
int16	CurrValue;
uint8	Percent;
	
	for(Channel = 0; Channel <= NumServos-1; Channel++)			// Iterate over all channels
	{
		CurrValue = ADC_GetResult16(Channel);					// Get ADC-value
		Percent   = CountsToPercent(CurrValue);					// to percent
		ChannelValues[Channel] = Percent;						// Save that
	}
}
//*******************************************************************************************//

CY_ISR(ChannelHandler)											// Handler for pulse ended
{
	PulseGen_ClearInterrupt(PulseGen_GetInterruptSource());		// Yes, clear interrupt
	System_Control_Write(MPXFire);								// Generate the MPX-Pulse
	ServoMUX_Write(CurrentChannel);								// Select Servo with Multiplexer
	if(CurrentChannel >= NumServos) 							// All channels done?
	{															// Yes
		System_Control_Write(ADCStart);							// Start the ADC
		return;													// Do not handle when outside Servo_Number
	}
	SetupPulse(CurrentChannel++);								// Program the pulse-width for the channel
}
//*******************************************************************************************//

CY_ISR(MainHandler)												// Handler for one cycle 
{
	MainTimer_ClearInterrupt(MainTimer_GetInterruptSource());	// Clear interrupt cause
	CurrentChannel = 0;											// Reset channel
	ServoMUX_Write(CurrentChannel);								// Select Servo with Multiplexer
	System_Control_Write(MPXFire);								// Generate MPX start pulse
	SetupPulse(CurrentChannel++);								// Program the pulse pause
}
/* [] END OF FILE */
